#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import click
from keyspwgenerator.tools import CmdLine, strength_checker


@click.command()
@click.option(
    "--length",
    "-l",
    "length",
    required=False,
    default=8,
    type=int,
    help="Length of desired password",
)
@click.option(
    "--remove",
    "-r",
    "remove",
    required=False,
    type=str,
    help="Values or characters to be excluded from the created \
        password (Enter values as a string... ex: 'j9_@Dy]'",
)
@click.option(
    "--no_repeats",
    "-n",
    "no_repeats",
    required=False,
    is_flag=True,
    default=False,
    help="Ensures there are no duplicate characters",
)
@click.option(
    "--check",
    "-c",
    "check",
    required=False,
    type=str,
    help="Checks string and returns password strength rating",
)
@click.option(
    "--strong",
    "-s",
    "strong",
    required=False,
    is_flag=True,
    help="Ignores length input and instead returns a strong password",
)
def main(
    length: int, remove: str, no_repeats: str, check: str, strong: bool
) -> None:  # main function
    """CLI app that creates a password for you!

    This function will create a random password for you. Default
    password length is 8 but can be changed by implementing the proper
    option. Refer to the --help option for more guides.

    Arguments:
        length: Desired length of password. Default is 8 characters.
        remove: Option that ensures characters are exempt (input as string)
        no_repeats: Flag that disables repeated characters
        check:  Reviews password input and provides a strength rating
    """

    # check password strength
    if check:
        click.echo(strength_checker(check))
    elif strong:
        pw = CmdLine(length=length, strong=strong)

        pw.create_password()

        click.echo(pw.get_password())
    else:
        # instantiate password object
        pw = CmdLine(length=length, strong=strong)

        pw.create_password()

        if remove:
            pw.remove_chars(remove)
        elif no_repeats:
            pw.remove_repeats()

        click.echo(pw.get_password())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from typing import List, Dict
import string
from random import shuffle
from collections import Counter
import secrets as sec
import re


class CmdLine:
    """
    Class that creates password as an object and can apply various methods
    for updates and validation

    Attributes:
        length: Desired length of password
        strength_flag: Flag declaring that the user prefers a strong password.
            If applied, this will ignore legth input and create a strong
            password instead

    Methods:
        create_password: Creates password from input list of chars
        remove_chars: Input is a string of unwanted characters.
            Method removes these characters from the password string
            and replaces them with new characters
        remove_repeats: Removes duplicate values from
            password string. Replaces duplicates with
            new characters from chars list

    """

    password: str = ""
    str_values: str = "".join(
        [
            string.ascii_lowercase,
            string.ascii_uppercase,
            string.digits,
            string.punctuation,
        ]
    )

    def __init__(self, length: int, strong: bool) -> None:
        self.length = length
        self.strong = strong

    def create_password(self) -> None:
        if self.strong:
            new_pw: str = sec.choice(CmdLine.str_values)

            counter: int = len(new_pw)

            strong_pw: str = "\33[30m\33[42m\33[1mSTRONG PASSWORD\033[0m"

            while strength_checker(new_pw) != strong_pw:
                shuffled_str_vals: List[str] = list(CmdLine.str_values)
                shuffle(shuffled_str_vals)
                CmdLine.str_values = "".join(shuffled_str_vals)
                new_char: str = sec.choice(CmdLine.str_values[counter % 4])
                new_pw += new_char
                counter += 1

            CmdLine.password = new_pw
        else:
            CmdLine.password = "".join(
                [sec.choice(CmdLine.str_values) for num in range(self.length)]
            )

    def remove_chars(self, removals: str) -> None:
        for char in removals:
            if char in CmdLine.password:
                CmdLine.str_values = CmdLine.str_values.replace(char, "")
                new_char = sec.choice(CmdLine.str_values)
                CmdLine.password = CmdLine.password.replace(char, new_char)

    def remove_repeats(self) -> None:
        pw_dict: Dict[str, int] = Counter(CmdLine.password)

        self.remove_chars("".join(list(pw_dict.keys())))

        while len(pw_dict) < self.length:
            new_char = sec.choice(CmdLine.str_values)
            pw_dict[new_char] += 1

        CmdLine.password = "".join(list(pw_dict.keys()))

    def get_password(self) -> str:
        return CmdLine.password


def strength_checker(val: str) -> str:
    score: int = 0
    str_values: List[str] = [
        string.ascii_lowercase,
        string.ascii_uppercase,
        string.digits,
        string.punctuation,
    ]

    pw_lens: List[int] = [8, 12, 16, 20]

    char_lens: List[int] = [1, 2, 3]

    for ele in pw_lens:
        if len(val) >= ele:
            score += 1

    for item in str_values:
        item_arr: List[str] = re.findall(f"[{item}]", val)
        for ele in char_lens:
            if len(item_arr) >= ele:
                score += 1

    if len(set(val)) == len(val):
        score += 2
    else:
        score -= 2

    if score >= 14:
        return "\33[30m\33[42m\33[1mSTRONG PASSWORD\033[0m"
    elif score < 14 and score >= 7:
        return "\33[30m\33[43m\33[1mMEDIUM PASSWORD STRENGTH\033[0m"
    else:
        return "\33[30m\33[41m\33[1mWEAK PASSWORD STRENGTH\033[0m"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['keyspwgenerator']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0']

entry_points = \
{'console_scripts': ['keys = keyspwgenerator.cli:main']}

setup_kwargs = {
    'name': 'keyspwgenerator',
    'version': '0.1.1',
    'description': 'CLI tool that creates passwords',
    'long_description': "# Keys (Password Generator)\n\n![python](https://img.shields.io/badge/Python-3.8%2C%203.9%2C%203.10%2C%203.11-blue)\n[![pip install](https://img.shields.io/badge/pip%20install-click-blue)](https://palletsprojects.com/p/click/)\n[![pypi](https://img.shields.io/badge/pypi-v0.1.0-orange)](https://pypi.org/project/keyspwgenerator/)\n[![license](https://img.shields.io/badge/License-MIT-blue)](https://opensource.org/license/mit/)\n[![Testing](https://github.com/jsattari/keys/actions/workflows/tests.yaml/badge.svg)](https://github.com/jsattari/keys/actions/workflows/tests.yaml)\n[![codecov](https://codecov.io/gh/jsattari/keys/branch/master/graph/badge.svg?token=8XQ4MXVR3M)](https://codecov.io/gh/jsattari/keys)\n\n---\n\n## Introduction to Keys\n\nKeys is a password generator that creates new passwords from uppercase letters, lowercase letters, digits, and special characters all within the Terminal.\n\n## Installation\n\n    pip3 install keyspwgenerator --user\n\n## Usage\n\n    Usage: keys [OPTIONS]\n\n        CLI app that creates a password for you!\n\n    Options:\n        -l, --length INTEGER  Length of desired password. Will default to 8\n                                if flag is not used\n        -r, --remove TEXT     Values or characters to be excluded from the created\n                                password (Enter values as a string... ex: 'j9_@Dy]'\n        -n, --no_repeats      Ensures there are no duplicate characters\n        -c, --check TEXT      Checks string and returns password strength rating\n        -s, --strong          Ignores length input and instead returns a strong\n                                password\n        --help                Show this message and exit.\n\n## Notes\n\n- Works with Python 3.8^\n",
    'author': 'John Sattari',
    'author_email': 'jsattari3@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jsattari/keys-pw-generator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
